package builder;

import java.awt.*;
import java.awt.event.*;
import java.io.*;


/**	Creates the two main windows for the builder interface.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/

public class Builder extends MyFrame{



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* The form and the workspace are created when this class is constructed.
*/
public Builder ()
{
  super("Demo: Work Space");
  testForm = new MyFrame("Demo: FORM");

  MenuBar mbar = new MenuBar();
  initFileMenu(mbar);
  initEditMenu(mbar);

  Map M = new Map();
  M.setBounds(new Rectangle( 10, 50, 465, 200));

  Map F = new Map();
  F.setBackground(Color.lightGray);
  F.setBounds(new Rectangle( 10, 50, 465, 200) );

  ToolBar toolBar = new ToolBar(M, F);

  initForm(F);
  initWorkSpace(toolBar,M,mbar);
}

private void initEditMenu(MenuBar mbar)
{
//---------------------------------------------------------------------------------
// The edit menu is not implemented yet. When it is implemented new actions should
// be added to the BuilderActionListener class.
//---------------------------------------------------------------------------------
		edit.add(new MenuItem("Undo"));
		edit.add(new MenuItem("-"));
		edit.add(new MenuItem("Cut"));
		edit.add(new MenuItem("Copy"));
		edit.add(new MenuItem("Paste"));
		mbar.add(edit);
		edit.setEnabled(false);
}
/** initFileMenu
  * this function creates the File Menu and adds it the menubar.
  * @param mbar is the menu bar to add the menu to.
*/
private void initFileMenu(MenuBar mbar)
{
//---------------------------------------------------------------------------------
// The actions that are disabled are not handled in the BuilderActionListener class.
// This functionality must be handled in the BuilderActionListener class when the
// functionality is added to the system.
//---------------------------------------------------------------------------------
  MenuItem newProject = new MenuItem("New Project");
  MenuItem openProject = new MenuItem("Open Project...");
  MenuItem saveProject = new MenuItem("Save Project");
  MenuItem saveAs = new MenuItem("Save As...");
  MenuItem submit = new MenuItem("Submit...");
  file.add(newProject);
  file.add(openProject);
  file.add(saveProject);
  file.add(saveAs);
  file.add(new MenuItem("-"));
  file.add(submit);
  file.add(new MenuItem("-"));
  file.add(new MenuItem("Quit"));
  mbar.add(file);
  openProject.setEnabled(false);
  saveProject.setEnabled(false);
  saveAs.setEnabled(false);
  submit.setEnabled(false);
}


/** initForm
  * This function sets the size, layout and location of the userd form.
  * @param F is the form map.
*/

private void initForm(Map F)
{
  testForm.setLayout(null);
  testForm.setSize(new Dimension(465, 250));
  testForm.setLocation(new Point(100, 100));
  testForm.setLayout(new BorderLayout( 5 , 5 ));
  testForm.add("Center", F);
  testForm.setVisible(true);
}
/** initWorkSpace
  * This function does the basic graphical layout for the workspace
  * @param toolBar is the toolbar to be added to the workspace.
  * @param M this is the map this is used to display moveable objects.
*/
private void initWorkSpace(ToolBar toolBar, Map M,MenuBar mbar)
{
  setSize(new Dimension(465, 250));
  setLocation(new Point(600, 100));
  setLayout(new BorderLayout( 5 , 5 ));
  add("North", toolBar);
  add("Center", M);
  setMenuBar(mbar);
  setListeners();
  setVisible(true);
}

/** 
  * This function adds the window listener for the workspace. It also adds the action listeners
  * for the menus.
*/
  public void setListeners()
  {
    this.addWindowListener(new BuilderWindowListener(this));
    BuilderActionListener menuListener = new BuilderActionListener(this);
    file.addActionListener(menuListener);
    edit.addActionListener(menuListener);
  }
/**
	* Creates a closed window event.
*/
  protected void quitWindowHelper()
  {
    processWindowEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSED));
  }

/**
	Quits program when Window is destroyed.
*/

  protected void closeWindowHelper()
  {
    testForm.setVisible(false);
    setVisible(false);
    System.exit(0);
  }
/** This is not fully implemented yet
*/
  protected void saveBuilder(String name)
  {
    ObjectOutputStream s = null;
    try
    {
     s = new ObjectOutputStream(new FileOutputStream(name));
    }catch(IOException e)
    {
      System.out.println("Duhhh, I can't save?");
      return;
    }
    try
    {
      s.writeObject(this);
    }catch(IOException e)
    {
      System.out.println("Me too dummm to write out");
    }
  }

/** This is not fully implemented yet
*/

  static public Builder loadBuilder(String name)
  {
    ObjectInputStream s = null;
    try
    {
     s = new ObjectInputStream(new FileInputStream(name));
    }catch(IOException e)
    {
      System.out.println("Duhhh, I can't load?");
      return null;
    }
    try
    {
      return (Builder)s.readObject();
    }catch(IOException e)
    {
      System.out.println("Me too dummm to read out");
    }catch(ClassNotFoundException e)
    {
      System.out.println("Class exception");
    }
    return null;
  }

/**	Run Builder.
*/
	public static void main(String args[]) 
	{
		f = null;
		f  = new Builder();
	}



// ********************* FIELDS

  static Frame f = null;
  public Frame testForm;
  Menu file = new Menu("File");
  Menu edit = new Menu("Edit");

}

