package builder;

import java.awt.*;
import java.io.*;



/**	A Node whose GUI representation is a Button.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class ButtonNode extends Node {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param text the name of the Node.
	* @param map the Map where the Node will paint itself onto.
	* @param locMap the location of the Node's GUI on the Panel.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	ButtonNode (String text, Map map, Point locMap, 
			Map form, Point locForm) {
		super(text, map, locMap, form, locForm);

		if (getMap().editMode() == false){
			runMode();
		}

		//setAttributes();
		nodeButton.addActionListener(new NodeActionListener(this));
		getFormLabel().setVisible(false);
	}

/**	Constructor used when placing the Node onto a location on the Form.
	* @param text the name of the Node.
	* @param map the Map where the Node will paint itself onto.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	public ButtonNode (String text, Map map, Map form, Point locForm) {
		this(text, map, new Point(1 , 1), form, locForm);

		moveIcon(new Point(getNodeNumber() * 5 , 
				   getNodeNumber() * 5));
	}

/**	Constructor used when placing the Node onto a location on the Map.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param locMap the location of the Node's icon on the map.
	* @param form the Panel on which the Node's GUI is placed.
*/
	public ButtonNode (String text, Map map, Point locMap, Map form) {
		this(text, map, locMap, form, new Point(1 , 1));

		move(new Point(	getNodeNumber() * 5 , 
				getNodeNumber() * 5));
	}



// ********************* METHODS


/**	paintPlaceHolder
	Defines the graphical representation of this Node.
	* @param g the graphics context.
*/
	public void paintPlaceHolder (Graphics g) {
		g.setColor(Color.red);
		g.drawString("<<  Button  >>", 10, 20);
		g.draw3DRect(1, 1, 120, 30, true);
	}


/**	getComponent  
	* @param c the component.
	* @return the Component of this Node
*/
	public Component getComponent(){
		return nodeButton;
	}

/** save
  * This function writes node information that is in required for the ButtonNode 
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os, String type)
  { 
    super.save(os,"ButtonNode");
    os.println(nodeButton.getLabel());
    os.println(END_OF_NODE);
  }

// ********************* FIELDS


	private Button nodeButton = new Button("");

}
