package builder;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**	Utility to modify the attributes of the different Nodes.
	* @author Arturo Falck.
	* @author Dennis Seay
	* @version 1.1 03/21/97
*/

public class ChangeAttributes extends Panel{

// ********************* CONSTRUCTORS
/**	Main Constructor.
	* @param n is the node that the attributes are valid for.
*/

  public ChangeAttributes(Node n)
  {
    super();
    node = n;

    choice = new Choice();
    ItemListener il = new ChangeAttributesItemListener(this);
    choice.addItemListener(il);

    // add base choices
    choice.addItem(NAME);
    choice.addItem(HIDE_NAME);
    choice.addItem(SHOW_NAME);

    if (n instanceof ButtonNode)
    {
      choice.addItem(BUTTON_NAME);
    }else if (n instanceof TextFieldNode)
    {
      choice.addItem(TEXT);
    }else if (n instanceof PullDownNode)
    {
      choice.addItem(ADD);
    }else if(n instanceof SliderNode)
    {
      choice.addItem(HIDE_RANGE);
      choice.addItem(SHOW_RANGE);
      choice.addItem(MIN_VALUE);
      choice.addItem(MAX_VALUE);
    }else if(n instanceof MailToNode)
    {
      choice.addItem(MAIL_SERVER);
      choice.addItem(USER_ID);
      choice.addItem(SEND_TO);
      choice.addItem(PORT);
    }
    choice.addItem(DELETE_NODE);
    setBounds(new Rectangle(node.getLocMapX(),node.getLocMapY(), 112, 32));
    choice.setBounds(new Rectangle(1,1,110,30));
    add(choice);
  }


// ********************* METHODS
/**	changeName
	Presets the actionMode to modify the Name of the Node and Icon.
	* @param n the node, whose Name will be modified.
*/
	public void changeName (Node n){
		String text = n.getFormLabel().toString();
		changeText(n, text);
	}

/**	getActionMode

  This function returns the current action mode.
  * @returns the actionMode for the current node*/

  public int getActionMode()
  {
    return actionMode;
  }

/** getChoice
  * @return the attribute menu for this node
*/
  public Choice getChoice()
  {
    return choice;
  }

/**	changeText
	Presents an editing window for text.
	* @param n the node, whose Name will be modified.
	* @param text that apears inside the editing window by default.
*/

	public void changeText (Node n, String text){
		textField = new TextField(text);
		textField.addActionListener(new ChangeAttributesActionListener(this));
		int width = text.length();
		width = (width < 5) ? 60 : width * 10;
		textField.setBounds(new Rectangle(1, 1, width, 30));
		textField.selectAll();
		add(textField);
      //        textField.requestFocus();
	}

/** setAction
  * sets the node's actionMode to a new value.
  * @param newAction is the new value for the action mode.
*/
  void setAction(int newAction)
  {
    actionMode = newAction;
  }

/** buttonAction
  * This function changes the button label for this component.
*/

  public void buttonAction()
  {
    getParent().remove(this);
    ((Button)node.getComponent()).
      setLabel(textField.getText());
  }

/** minAction
  * This function sets a new minimum value for a slider
*/
  public void minAction()
  {
    Integer min = null;
    Integer max = null;
    try
    {
      min = new Integer(textField.getText());
      max = new Integer(((SliderNode)node).getMaxLabel().toString());
    }catch(NumberFormatException e)
    {
      textField.setText("Error");
      textField.selectAll();
      return;
    }
    if(min.intValue() >= max.intValue())
    {
      textField.setText("Error");
      textField.selectAll();
      return;
    }
    getParent().remove(this);
    ((SliderNode)node).setMin(min.intValue());
  }

/** maxAction
  * This function sets the max value for a slider bar.
*/
  public void maxAction()
  {
    Integer min = null;
    Integer max = null;
    try
    {
      max = new Integer(textField.getText());
      min = new Integer(((SliderNode)node).getMinLabel().toString());
    }catch(NumberFormatException e)
    {
      textField.setText("Error");
      textField.selectAll();
      return;
    }
    if(min.intValue() >= max.intValue())
    {
      textField.setText("Error");
      textField.selectAll();
      return;
    }
    getParent().remove(this);
    ((SliderNode)node).setMax(max.intValue());
  }

/** tfieldAction
  * This function sets the data for a text field (input and output)
*/

  public void tfieldAction()
  {
    getParent().remove(this);
    ((TextField)node.getComponent()).
      setText(textField.getText());
  }  

/** pdMenuAction
  * This function adds a choice to the pulldown menu
*/

  public void pdMenuAction()
  {
    getParent().remove(this);
    ((Choice)node.getComponent()).
      addItem(textField.getText());
  }
 
/** nameAction
  * This function sets the label for a given node
*/
 
  public void nameAction()
  {
    getParent().remove(this);
    node.getFormLabel().
      setLabel(textField.getText());
    node.getIcon().
      getIconLabel().
      setLabel(textField.getText());       
    choice.setVisible(true);
  }

/** mailServerAction
  * This function changes the server that mail is being sent to.
*/

  public void mailServerAction()
  {
    getParent().remove(this);
    ((MailToNode)node).setMailServer(textField.getText());
  }

/** mailServer
  * This function initiates the changeText for the mailServer.
*/

  public void mailServer()
  {
    String text = ((MailToNode)node).getMailServer();
    changeText(node, text);
  } 

/** userIDAction
  * This function changes the id that mail is being sent from.
*/

  public void userIDAction()
  {
    getParent().remove(this);
    ((MailToNode)node).setUserID(textField.getText());
  }

/** userID
  * This function initiates the changeText for the userID.
*/

  public void userID()
  {
    String text = ((MailToNode)node).getUserID();
    changeText(node, text);
  }

/** sendToAction
  * This function changes the person that mail is being sent to.
*/

  public void sendToAction()
  {
    getParent().remove(this);
    ((MailToNode)node).setSendTo(textField.getText());
  }

/** SendTo
  * This function initiates the changeText for the sendTo.
*/

  public void sendTo()
  {
    String text = ((MailToNode)node).getSendTo();
    changeText(node, text);
  }

/** portAction
  * This function changes the port that the mail is sent to.
*/

  public void portAction()
  {
    Integer portNum = null;  
    try  
    { 
      portNum = new Integer(textField.getText());
    }catch(NumberFormatException e)
    {
      textField.setText("Error");
      textField.selectAll();
      return;
    }

    getParent().remove(this);
    ((MailToNode)node).setPort(portNum.intValue());
  }

/** port
  * This function initiates the changeText for the port.
*/

  public void port()
  {
    Integer portNum = new Integer(((MailToNode)node).getPort());
    String text = new String(portNum.toString());
    changeText(node, text);
  }

/** changeName
  * This function allows changeName to be called without allow access to private data.
*/

  public void changeNameHelper()
  {
    changeName(node);
  }

/** hideName
  * This function hides the nodes label.
*/

  public void hideName()
  {
    getParent().remove(this);
    node.getFormLabel().setVisible(false);
  }

/** showName
  * This function shows the nodes label.
*/

  public void showName()
  {
    getParent().remove(this);
    node.getFormLabel().setVisible(true);
  }

/** hideRange
  * This function hides the nodes range.
*/

  public void hideRange()
  {
    getParent().remove(this);
    ((SliderNode)node).getMinLabel().setVisible(false);
    ((SliderNode)node).getMaxLabel().setVisible(false);
  }

/** showName
  * This function shows the nodes range.
*/

  public void showRange()
  {
    getParent().remove(this);
    ((SliderNode)node).getMinLabel().setVisible(true);
    ((SliderNode)node).getMaxLabel().setVisible(true);
  }

/** buttonName
  * This function initiates the changeText for the button.
*/

  void buttonName()
  {
    String text = ((Button)node.getComponent()).
      getLabel();
    changeText(node, text);
  }

/** minValue
  * This function initiates the changeText for the minValue.
*/

  void minValue()
  {
    String text = ((SliderNode)node).getMinLabel().toString();
    changeText(node, text);
  }

/** maxValue
  * This function initiates the changeText for the maxValue.
*/

  void maxValue()
  {
    String text = ((SliderNode)node).getMaxLabel().toString();
    changeText(node, text);
  }

/** setTFieldText
  * This function initiates the changeText for the textField.
*/

  void setTFieldText()
  {
    String text = ((TextField)node.getComponent()).getText();
    changeText(node, text);
  }

 /** deleteNode
 */
  void deleteNode()
  {
    Vector wVector = null;

    if(!node.getPaint())
    {
      node.removeComponent(node.getComponent());
    }
    if(node instanceof SliderNode)
    {
      ((SliderNode)node).getMinLabel().setVisible(false);
      ((SliderNode)node).getMaxLabel().setVisible(false);
    }
    node.getFormLabel().setVisible(false);
    getParent().remove(this);
    ((Icon)node.getIcon()).removeIcon();
    System.out.println(((Icon)node.getIcon()).getMap().nodesVector.contains(node) );
    ((Icon)node.getIcon()).getMap().nodesVector.removeElement(node);
    ((Map)node.getMap()).remove(node);

    wVector = ((Icon)node.getIcon()).getMap().wiresVector;

    for(int x=0;x<wVector.size();x++)
    {
      System.out.println(x + ": Wire number");
      if( ((Wire)wVector.elementAt(x)).getInput().getNodeNumber()==node.getNodeNumber() )
      {
        wVector.removeElementAt(x);
        System.out.println(x + ": removing input");
        x--;
      }
      if( ((Wire)wVector.elementAt(x)).getOutput().getNodeNumber()==node.getNodeNumber() )
      {
        wVector.removeElementAt(x);
        System.out.println(x + ": removing output");
        x--;
      }
      System.out.println("Node: " + x);
    }
    node.getIcon().getMap().repaint();
    System.out.println("Delete node activated");
  }

/** addPDMenuItem
  * This function initiates the changeText for the pulldown menu.
*/

  void addPDMenuItem()
  {
    changeText(node, "");
  }
// ********************* FIELDS




	public final static int	NO_ACTION          = 0;
	public final static int	TEXT_ACTION        = 1;
	public final static int	MIN_VAL_ACTION     = 2;
	public final static int	MAX_VAL_ACTION     = 3;
	public final static int	NAME_ACTION        = 4;
	public final static int	BUTTON_NAME_ACTION = 5;
	public final static int	PDMENU_ADD_ACTION  = 6;
	public final static int MAIL_SERVER_ACTION = 7;
	public final static int	USER_ID_ACTION     = 8;
	public final static int	SEND_TO_ACTION     = 9;
	public final static int PORT_ACTION        = 10;
        public final static int DELETE_NODE_ACTION = 11;

	public final static String	NAME        = "Name";
	public final static String	TEXT        = "Text";
	public final static String	ADD         = "Add Item";
	public final static String	HIDE_NAME   = "Hide Name";
	public final static String	SHOW_NAME   = "Show Name";
	public final static String      MIN_VALUE   = "Set minimum value";
	public final static String      MAX_VALUE   = "Set maximum value";
	public final static String	BUTTON_NAME = "Button Name";
	public final static String	HIDE_RANGE  = "Hide Range";
	public final static String	SHOW_RANGE  = "Show Range";
	public final static String 	MAIL_SERVER = "Set Mail Server";
	public final static String 	USER_ID     = "Set Return Address";
	public final static String 	SEND_TO     = "Set Destination";
	public final static String 	PORT        = "Server Port";
        public final static String      DELETE_NODE = "Delete Node";
	private int         actionMode;
	private Node        node;
	private Choice      choice;
	private TextField   textField;
}
