package builder;

import java.awt.*;




/**	The Flow Control representation of the Node.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class Icon extends Moveable {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param label name associated with the Icon.
	* @param m the Map where the Icon is placed.
	* @param l initial location of the Icon in the Map.
	* @param parent The Node to which this Icon belongs.
*/
	public Icon (String label, Map m, Point l, Node parent){
		this.parent = parent;
		setMap(m);
		setLocMap(l);
                setListener(this);
		initialize(33, 23);

		iconLabel = new Name (	label,
					m,
					new Point (
						l.x +5 ,
						l.y + this.getBounds().height ));

	}

/**	Blank Constructor.
*/
	public Icon (){
		this ("no Label", new Map(), new Point(0,0), null);
	}



// ********************* METHODS
/* setListener
  * This function sets the default listener for a moveable object
*/

  public void setListener(Moveable obj)
  {
    addMouseListener(new IconMouseListener(obj));
    addMouseMotionListener(new MoveableMouseMotionListener(obj));
  }

/** removeWire
 *  removes Wire from the vector.
*/
  public void removeWire(Wire wire)
  {
    getMap().wiresVector.removeElement(wire);
  }

/**	removeIcon
	Removes the Icon and it's label from the Map.
*/
	public void removeIcon(){
		getMap().remove(this);
		getMap().remove(iconLabel);
	}

/**	getIconLabel
	* @return the label associated with this Icon.
*/
	public Name getIconLabel(){
		return iconLabel;
	}

/**	getParentNode
	* @return the Node that this Icon represents.
*/
	public Node getParentNode(){
		return parent;
	}

/**	setMoving
	sets the Icon's own moving status and the moving status of it's label.
	* @param status true if moving, flase if not moving.
*/
	public void setMoving(boolean status) {
		iconLabel.lock(status);
		iconLabel.setMoving(status);
		super.setMoving(status);
	}

/**	move
	Moves itself and it's label.
	* @param delta how much to move the Icon in the x and y directions.
*/
	public void move(Point delta){
		iconLabel.move(delta);
		super.move(delta);
	}

/**	lock
	sets the it's own lock status and the lock status of it's label.
	* @param status true if moving, flase if not moving.
*/
	public void lock(boolean status){
		iconLabel.lock(status);
		super.lock(status);
	}


/**	paint
	Defines the icon for the map representation of this Node.
	* @param g the graphics context.
*/
	public void paint (Graphics g) {
		g.draw3DRect(1, 1, 30, 20, true);
		g.draw3DRect(6, 5, 20, 12, true);
	}



// ********************* Event METHODS
/* rightMouseClick
  * This function is called when the right mouse was clicked. It brings up a new
  * attribute menu.
*/

   public void rightMouseClick()
   {
      try
      {
        ChangeAttributes c =
        new ChangeAttributes(getParentNode());
        c.setLocation(	getLocMapX() +
          this.getBounds().width /2,
          getLocMapY() +
          this.getBounds().height/2);
        getMap().add(c);
        getMap().setVisible(true);
      }
      catch (NullPointerException e){}
    }

/** wiringLeftButtonPressed
  * This is called when the left button is pressed while in wiring mode.
  * if connects the two selected icons together.
*/
   public void wiringLeftButtonPressed()
   {
      //getMap().createWire(getParentNode());
   }
/** wiringLeftButtonReleased
  * This is called when the left button is released while in wiring mode.
  * if connects the two selected icons together.
*/

  public void wiringLeftButtonReleased()
  {
    getMap().createWire(getParentNode());
  }

// ********************* FIELDS


	private Name iconLabel;
	private Node parent;



}
