package builder;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.net.*;
/** Author Dennis Seay
  * Version 1.1
*/

public class LoadNodes
{
  BufferedReader is        = null;
  URL            url       = null;//url from where the file is to be read from
  Map            workSpace = null;
  Map            form      = null;
  String         header    = "";
  String         nodeType  = "";
  boolean        visible   = false;
  String         name      = "";
  String         mode      = "";
  Point          nodeLoc   = null;
  Point          iconLoc   = null;
  Point          labelLoc  = null;
  Point          iconLabelLoc = null;
  WorkSpace      workspace = null;
  FormSpace      formspace = null;
/** Main Constructor
*/
  public LoadNodes(Map ws, Map fm, String fileName, FormSpace forms,WorkSpace work)
  {
    workSpace = ws;
    form = fm;
	try {url = new URL(fileName);}
	    catch(IOException e) {
	    e.printStackTrace();
	}
		try {is = new BufferedReader(new InputStreamReader(url.openStream()));}
			    catch(IOException e) {
	    e.printStackTrace();
	}

//    is = openLoadFile(fileName);
    workspace = work;
    formspace = forms;
  }

/** readDimension
  * This function returns a Dimension
*/
Dimension readDimension() throws IOException
{
  int height = 0;
  int width  = 0;
  String line = "";
  Dimension dimension = null;

  line = is.readLine();
  StringTokenizer s = new StringTokenizer(line);

  height = (new Integer(s.nextToken())).intValue();
  width = (new Integer(s.nextToken())).intValue();
  //System.out.println("Save x: " + x + "Save y: " + y);
  dimension = new Dimension(width,height);
  return dimension;
}

/** readPoint
  * This function returns a point
*/
Point readPoint() throws IOException
{
  int x = 0;
  int y = 0;
  String line = "";
  Point point = null;

  line = is.readLine();
  StringTokenizer s = new StringTokenizer(line);

  x = (new Integer(s.nextToken())).intValue();
  y = (new Integer(s.nextToken())).intValue();
  //System.out.println("Save x: " + x + "Save y: " + y);
  point = new Point(x,y);
  return point;
}
/** closeLoadFile
  * This function closes any open files that were being loaded to.
*/
  public void closeLoadFile()
  {
    try
    {
      is.close();
    }catch(IOException e)
    {
    }
  }



/** loadLocation
  * This function loads the location of the workspace and formspace.
*/

  public void loadLocation()
  {
    Point p = null;
    Dimension d = null;

    try
    {
      header = is.readLine();
      if(!header.equals(SaveNodes.BEGIN_SPACE_DATA))
      {
        System.out.println("Error: invalid file format");
        return;
      }
      p = readPoint();
      d = readDimension();
      formspace.setLocation(p);
      formspace.setSize(d);
      form.setSize(d);
      p = readPoint();
      d = readDimension();
      workspace.setLocation(p);
      workspace.setSize(d);
      workSpace.setSize(d);

      workspace.repaint();
      formspace.repaint();

      String data = is.readLine();
      if(!data.equals(SaveNodes.END_SPACE_DATA))
      {
        System.out.println("Error: inavalid file format");
      }
    }catch (IOException e)
    {
      System.out.println("Error: can not load file");
    }

  }



/** loadWire
  * This function loads the node names to connect.
*/

  public void loadWire()
  {
    int inputNode  = 0;
    int outputNode = 0;
    String wireNodes = "";

    try
    {
      header = is.readLine();
      if(!header.equals(Wire.BEGIN_WIRE_DATA))
      {
        System.out.println("Error: invalid file format");
        return;
      }
      while(true)
      {
        String wireNode = "";
        wireNodes = is.readLine();

        if(wireNodes.equals(Wire.END_WIRE_DATA))
        {
          return;
        }

        StringTokenizer s = new StringTokenizer(wireNodes);

        inputNode  = (new Integer(s.nextToken())).intValue() - 1;
        outputNode = (new Integer(s.nextToken())).intValue() - 1;
        //System.out.println("Save inputNode: " + inputNode +
        //                   "Save outputNode: " + outputNode);
        new Wire((Node)workSpace.nodesVector.elementAt(inputNode),
                 (Node)workSpace.nodesVector.elementAt(outputNode));
        form.setBackground(Color.lightGray);
      }
    }catch (IOException e)
    {
      System.out.println("Error: can not load file");
    }

  }

/** load
  * This function loads a save file
*/
  public void load()
  {
  }
/** loadNode
  * This function loads the information to recreate a node.
*/

  public void loadNode()
  {
    try
    {
      header = is.readLine();
      if(!header.equals(Node.BEGIN_NODE_DATA))
      {
        System.out.println("Error: invalid file format");
        return;
      }
      header = Node.END_OF_NODE;
      while(header.equals(Node.END_OF_NODE))
      {
        nodeType = is.readLine();
        if(nodeType.equals(Node.END_NODE_DATA))
        {
          return;
        }
        nodeLoc = readPoint();
        name = is.readLine();
        labelLoc = readPoint();
        visible = (new Boolean(is.readLine())).booleanValue();
        mode = is.readLine();
        iconLoc = readPoint();
        iconLabelLoc = readPoint();
        createNode();
      }
    }catch (IOException e)
    {
      System.out.println("Error: can not load file");
    }

  }

  void createNode() throws IOException
  {
    Node n = null;
    Name f = null;
    if(nodeType.equals("ButtonNode"))

    {
      n = new ButtonNode(name,workSpace,iconLoc,form);
      ((Button)n.getComponent()).setLabel(is.readLine());
      header = is.readLine();
    }else if(nodeType.equals("MailToNode"))
    {
      n = new MailToNode(name,workSpace,iconLoc,form);
      ((MailToNode)n).setMailServer(is.readLine());
      ((MailToNode)n).setUserID(is.readLine());
      ((MailToNode)n).setSendTo(is.readLine());
      ((MailToNode)n).setPort((new Integer(is.readLine())).intValue());
      header = is.readLine();
    }else if(nodeType.equals("PullDownNode"))
    {
      String nextData = "";
      n = new PullDownNode(name,workSpace,iconLoc,form);
      nextData = is.readLine();
      while(!nextData.equals(Node.END_OF_NODE))
      {
        ((Choice)n.getComponent()).addItem(nextData);
        nextData = is.readLine();
      }
      header = nextData;
    }else if(nodeType.equals("SliderNode"))
    {
      n = new SliderNode(name,workSpace,iconLoc,form);
      ((Scrollbar)n.getComponent()).setValue((new Integer(is.readLine())).intValue());
      ((SliderNode)n).setMin((new Integer(is.readLine())).intValue());
      ((SliderNode)n).getMinLabel().setPoint(readPoint());
      if(is.readLine().equals("false"))
      {
        System.out.println("false");
        ((SliderNode)n).getMinLabel().setVisible(false);
        ((SliderNode)n).getMaxLabel().setVisible(false);
      }
      ((SliderNode)n).setMax((new Integer(is.readLine())).intValue());
      ((SliderNode)n).getMaxLabel().setPoint(readPoint());
      if(is.readLine().equals("false"))
      {
        ((SliderNode)n).getMinLabel().setVisible(false);
        ((SliderNode)n).getMaxLabel().setVisible(false);

      }
      header = is.readLine();
    }else if(nodeType.equals("TextFieldNode"))
    {
       n = new TextFieldNode(name, workSpace, iconLoc ,form);
       if(mode.charAt(0)=='o')
       {
         n.setIoOption(Node.OUTPUT);
       }
       ((TextField)n.getComponent()).setText(is.readLine());
       header = is.readLine();
    }
    f = n.getFormLabel();
    n.setPoint(nodeLoc);
    f.setPoint(labelLoc);
    f.setVisible(visible);
    f = ((Icon)n.getIcon()).getIconLabel();
    f.setPoint(iconLabelLoc);
  }
}
/** openLoadFile
  * This function opens the file for saving if it is a valid file name/file.
*/
/*  BufferedReader openLoadFile(String fileName)
  {
    BufferedReader input;
    File f = new File(fileName);
    if(f.exists())
    {
      if(f.isFile())
      {
        System.out.println("Loading file");
      }else
      {
        System.out.println("Can not read a directory");
        return null;
      }
    }else
    {
      System.out.println("File does not exist");
      return null;
    } 
    try  
    { 
     input = new BufferedReader(new FileReader(f));
    }catch(IOException e)
    {
      System.out.println("Unable to load");
      return null;
    }
    return input;
  }
}
*/