package builder;

import java.awt.*;
import java.util.Vector;
import SMTPClient.Client;
import java.io.*;

/**	A Node whose GUI representation is a Button.
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class MailToNode extends Node {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param text the name of the Node.
	* @param map the Map where the Node will paint itself onto.
	* @param locMap the location of the Node's GUI on the Panel.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	MailToNode (String text, Map map, Point locMap, 
			Map form, Point locForm) {
		super(text, map, locMap, form, locForm);

		if (getMap().editMode() == false){
			runMode();
		}

		//setAttributes();
		nodeButton.addActionListener(new NodeActionListener(this));
		getFormLabel().setVisible(false);
	}

/**	Constructor used when placing the Node onto a location on the Form.
	* @param text the name of the Node.
	* @param map the Map where the Node will paint itself onto.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	public MailToNode (String text, Map map, Map form, Point locForm) {
		this(text, map, new Point(1 , 1), form, locForm);

		moveIcon(new Point(getNodeNumber() * 5 , 
				   getNodeNumber() * 5));
	}

/**	Constructor used when placing the Node onto a location on the Map.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param locMap the location of the Node's icon on the map.
	* @param form the Panel on which the Node's GUI is placed.
*/
	public MailToNode (String text, Map map, Point locMap, Map form) {
		this(text, map, locMap, form, new Point(1 , 1));

		move(new Point(	getNodeNumber() * 5 , 
				getNodeNumber() * 5));
	}

/** nodeAction
  * this function handles Node events
  * @param arg is the node that generated the event
*/
  public void nodeAction(String arg)
  {  
    argument = arg;
    System.out.println("Node action occured: " + argument);

    Client c = null;
    c = new Client(mailServer,25);
    if(c.helo())
      System.out.println("OK - HELO");
    if(c.sendFrom(userID))
      System.out.println("OK - sendFrom");
    if(c.sendTo(sendTo))
      System.out.println("OK - sendTo");
    if(c.startData())
      System.out.println("OK - startData");
    for(int x = 0;x<dataVector.size();x++)
    {
    if(c.data((String)dataVector.elementAt(x)))
      System.out.println("OK - data 1");
    }
    if(c.endData())
      System.out.println("OK - endData");
    if(c.quit())
      System.out.println("OK - quit");
  }

// ********************* METHODS


/**	paintPlaceHolder
	Defines the graphical representation of this Node.
	* @param g the graphics context.
*/
	public void paintPlaceHolder (Graphics g) {
		g.setColor(Color.red);
		g.drawString("<<  Mail To  >>", 10, 20);
		g.draw3DRect(1, 1, 120, 30, true);
	}


/**	getComponent  
	* @param c the component.
	* @return the Component of this Node
*/
  public Component getComponent()
  {
    return nodeButton;
  }

  private int indexOf(Node c)
  {
    int size = wiresVector.size();
    for(int x = 0;x < size;x++)
    {
      if(((Wire)wiresVector.elementAt(x)).getInput()==c)
      {
        return x;
      }
    }
    return -1;
  }
/**     incommingAction
	* This function keeps the current data to send when a submit comes in.
        * @return true.
*/
  public boolean incommingAction(Node caller)
  {
    int index = indexOf(caller);
    System.out.println(index + ": " + caller.getArgument());
    if(dataVector.size() <= index)
    {
      while((index-dataVector.size())>=0)
      {
        dataVector.addElement("");
      }
    } 
    dataVector.setElementAt(caller.getArgument(),index);
    System.out.println("Data at " + index +": " + (String)dataVector.elementAt(index));
    return true;
  }

/**     setUserID
	* This function changes the ID attribute
*/

  public void setUserID(String nid)
  {
    userID = nid;
  }

/**     getUserID
	*@return This function returns the ID attribute
*/

  public String getUserID()
  {
    return userID;
  }

/**     setMailServer
	* This function changes the Mail Server attribute
*/

  public void setMailServer(String ns)
  {
    mailServer = ns;
  }

/**     getMailServer
	*@return This function returns the Mail Server attribute
*/

  public String getMailServer()
  {
    return mailServer;
  }

/**     setSendTo
	* This function changes who the mail goes to
*/

  public void setSendTo(String ns)
  {
    sendTo = ns;
  }

/**     getSendTo
	*@return This function returns who the mail goes to
*/

  public String getSendTo()
  {
    return sendTo;
  }

/**     setPort
	* This function changes the port the data goes to.
*/

  public void setPort(int np)
  {
    port = np;
  }

/**     getPort
	* This function returns the port the data goes to.
*/

  public int getPort()
  {
    return port;
  }


/** save
  * This function writes node information that is in required for the MailToNode
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os, String type)
  {
    super.save(os,"MailToNode");
    os.println(mailServer);
    os.println(userID);
    os.println(sendTo);
    os.println(port);
    os.println(END_OF_NODE);
  }

// ********************* FIELDS


  private Button nodeButton  = new Button("Submit Form");
  private String mailServer  = "mail.vt.edu";
  private String userID      = "builder";
  private String sendTo      = "dseay@vt.edu";
  private int    port        = 25;
          Vector dataVector  = new Vector(100);
}
