package builder;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.io.*;




/**	A surface where either components or icons are placed and wires are
	drawn.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class Map extends Panel {



// ********************* CONSTRUCTORS


/**	Main Constructor.
*/
	public Map () {
		super();

		setLayout(null);
		setBackground(Color.white);
		setSize (new Dimension(500 , 400));
		setLocation (new Point(10, 50));
		addMouseListener(new MapMouseListener(this));
		addMouseMotionListener(new MapMouseMotionListener(this));
	}

/**	Constructor that includes the loading of the file.
	* @param file_name a file containing the Node and Wire Vectors.
*/
	public Map (File file_name) {
		this();

		// some file manipulations to load the Vectors.
	}



// ********************* METHODS


/**	addNode
	Adds Node to the vector.
	* @param node the node to add to the vector.
	* @return the index of the node added.
*/
	public int addNode(Node node) {
		nodesVector.addElement(node);
		return nodesVector.size();
	}

/**	addWire
	Adds Wire to the vector.
	* @param wire the wire to add to the vector.
	* @return the index of the wire added.
*/
	public int addWire(Wire wire){
		wiresVector.addElement(wire);
		return wiresVector.size();
	}


/**	editMode
	A method to set the Edit Mode.
	* @param mode true for Edit Mode, false for Run Mode.
*/
	public void editMode(boolean mode) {
		editing = mode;
		if (nodesVector.isEmpty() != true){
			try{
			   if (editing == true)
				for (int i=0; i<nodesVector.size(); i++)
				   ((Node)nodesVector.elementAt(i)).editMode();
			   else
				for (int i=0; i<nodesVector.size(); i++)
				   ((Node)nodesVector.elementAt(i)).runMode();
			}
			catch (ArrayIndexOutOfBoundsException e) {}
		}
		invalidate();
		validate();
	}

/**	editMode
	* @return the Edit Mode of this Map.  
	true for Edit Mode, false for Run Mode.

*/
	public boolean editMode(){
		return editing;
	}


/**	lockAll
	Locks all components of this Map in place.
	* @param status true for Locked, false for Unlocked.
*/
	public void lockAll(boolean status) {
		if (nodesVector.isEmpty() != true){
			if (status)
			   for (int i=0; i<nodesVector.size(); i++)
				((Node)nodesVector.elementAt(i)).
					getIcon().lock(true);
			else
			   for (int i=0; i<nodesVector.size(); i++)
				((Node)nodesVector.elementAt(i)).
					getIcon().lock(false);
		}
	}


/**	wireAll
	Presets all Icons to be ready to accept wires.
	* @param status true for wiring enabled, false for disabled.
*/
  public void wireAll(boolean status) 
  {
    if(nodesVector.isEmpty() != true)
    {
      if (status)
      {
        ((Frame)getParent()).setCursor(new Cursor(Cursor.HAND_CURSOR));

        for (int i=0; i<nodesVector.size(); i++)
          ((Node)nodesVector.elementAt(i)).
        getIcon().wireMouse(true);
      }
      else
      {
        ((Frame)getParent()).setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

        for (int i=0; i<nodesVector.size(); i++)
          ((Node)nodesVector.elementAt(i)).
        getIcon().wireMouse(false);
      }
    }
  }

/**	movingThis
	Tells this map which Object is moving.
	* @param target the Moveable Object that is being moved at the time.
	* @return true.
*/
	public boolean movingThis(Moveable target){
		movingObject = target;
		return true;
	}

/**	movingThis
	Tells this map that NO Object is moving.
	* @return true.
*/
	public boolean movingThis(){
		movingObject = null;
		return true;
	}

  public boolean isMoving()
  {
    if ((movingObject != null) && (movingObject.isMoving()))
    {
      return true;
    }
    return false;
  }

/**	createWire
	Tells this map that a Wire is being created... This method is called 
	twice, with two different nodes.  The first time it is called it sets 
	the starting Node for the wire, the second time, it sets the ending
	Node for the wire and creates the Wire.
	* @param target the Node that the Wire is anchored to.
*/
	public void createWire(Node target){
		if (startWire == null){
			//System.out.println("start at " + target);
			startWire = target;
		}
		else{
			if (target != startWire){
				//System.out.println("end at " + target);
				Wire newWire = new Wire (startWire, target);
				startWire = null;
			}
		}

	}


/**	paint
	Defines the graphical representation of the wires on this map.
	* @param g the graphics context.
*/
	public void paint (Graphics g) {
		if (wiresVector.isEmpty() != true){
			for (int i=0; i<wiresVector.size(); i++)
				((Wire)wiresVector.elementAt(i)).paintWire(g);
		}
	}



// ********************* Event METHODS

/**  leftButtonReleased
  * This function moves the object to its final lacation on the map
  * @param x is the x location on the map
  * @param y is the y location on the map
*/

  public void leftButtonReleased(int x, int y)
  {
    movingObject.move (new Point(
      x - movingObject.getLocation().x
        - movingObject.getZero().x  ,
      y - movingObject.getLocation().y
        - movingObject.getZero().y));
    movingObject.leftButtonReleased();

    if ((startWire != null) && (getComponentAt(x,y) instanceof Icon))
    {
      createWire(((Icon)getComponentAt(x,y)).getParentNode());
    }
  }


/**  leftDraggedReleased
  * This function moves the object to its new lacation on the map 
  * @param x is the x location on the map 
  * @param y is the y location on the map 
*/

  public void leftMouseDragged(int x, int y)
  {
    movingObject.move (new Point(
      x - movingObject.getLocation().x
        - movingObject.getZero().x  ,
      y - movingObject.getLocation().y
        - movingObject.getZero().y));
  }



// ********************* FIELDS


		Vector		nodesVector = new Vector();
		Vector		wiresVector = new Vector();
	private boolean		editing = true;
	private Moveable 	movingObject;
	private Node		startWire;
}
