package builder;

import java.awt.*;




/**	An Abstraction.  This is an Object that has the ability to be dragged on
	a Map.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class Moveable extends Panel   {



// ********************* CONSTRUCTORS

  public Moveable()
  {
    //setListener(this);
  }

// ********************* METHODS


/**	setListener
	Sets the listener with the appropriate subclass of moveable to listen
	for.
	* @param m the Map.
*/
  public void setListener(Moveable obj)
  {
    addMouseListener(new MoveableMouseListener(obj));
    addMouseMotionListener(new MoveableMouseMotionListener(obj));
  }
/**	setMap
	Sets the map to which this Moveable Object belongs to.
	* @param m the Map.
*/
	public void setMap (Map m) {
		map = m;
	}

/**	getMap
	* @return the Map that this Moveable Object belongs to.
*/
	public Map getMap () {
		return map;
	}


/**	setLocMap
	Sets the location of this Moveable Object in the Map.
	* @param x coordinate in the Map coordinates.
	* @param y coordinate in the Map coordinates.
*/
	public void setLocMap (int x, int y) {
		locMap = new Point(x, y);
	}

/**	setLocMap
	Sets the location of this Moveable Object in the Map.
	* @param p the location in the Map coordinates.
*/
	public void setLocMap (Point p) {
		this.setLocMap(p.x, p.y);
	}

/**	getLocMap
	* @return the location of this Moveable Object in the Map.
*/
	public Point getLocMap () {
		return locMap;
	}

/**	getLocMapX
	* @return the x coordinate of this Moveable Object in the Map.
*/
	public int getLocMapX () {
		return locMap.x;
	}

/**	getLocMapY
	* @return the y coordinate of this Moveable Object in the Map.
*/
	public int getLocMapY () {
		return locMap.y;
	}


/**	initialize
	Initializes and places this Moveable Object on the Map.
	* @param backgroundColor the Background color of this Moveable Object.
	* @param size the initial Dimension of this Moveable Object.
*/
	public void initialize (Color backgroundColor, Dimension size) {
		setBackground(backgroundColor);
		setBounds(new Rectangle( getLocMapX (), getLocMapY (),
			 size.width, size.height ));
		getMap().add(this);
	}

/**	initialize
	Initializes and places this Moveable Object on the Map.
	* @param size the initial Dimension of this Moveable Object.
*/
	public void initialize (Dimension size) {
		this.initialize(getMap().getBackground(), size);
	}

/**	initialize
	Initializes and places this Moveable Object on the Map.
	* @param width the initial Width of this Moveable Object.
	* @param height the initial Height of this Moveable Object.
*/
	public void initialize (int width, int height) {
		this.initialize(new Dimension(width, height));
	}


/**	getDelta
	* @return the distance and direction that this Moveable Object has
	moved.
*/
	public Point getDelta () {
		return delta;
	}


/**	getZero
	* @return the Location where this Moveable Object was before moving.
*/
	public Point getZero () {
		return zero;
	}

/**	setZero
	Sets the Location where this Moveable Object was before moving.
*/
	public void setZero (Point set) {
		zero = set;
	}



/**	lock
	Locks this Moveable in place.
	* @ param status true if locked, false if unlocked.
*/
	public void lock(boolean status){
		ignoreMouse(status);
	}


/**	move
	Moves this Moveable Object.
	* @ param delta how much to move the object.
*/
	public void move(Point delta) {
		setLocMap (new Point( 	getLocMapX () + delta.x ,
						getLocMapY () + delta.y ));

		super.setLocation(new Point(locMap.x, locMap.y));
	}	
/** setPoint
  * Moves the object the exact point.
  * @param moveTo is the absolute coordinate to move to.
*/
  public void setPoint(Point moveTo)
  {
    setLocMap(moveTo);
    super.setLocation(moveTo);
  }
/**	isMoving
	* @return true if this Moveable Object is moving, false when not.
*/
	public boolean isMoving() {
		return moving;
	}

/**	This is a piece of code that I am not proud of.  The purpose of the 
	method is to set the moving variable for when objects are dragged along
	or moved but I am also using it to update the whole screen at the end of
	a move so that wires are repainted in the flow map.
	* @ param state true if moving, false if not moving.
*/
	public void setMoving(boolean state) {
		if (moving != state)
			if ( state == true)
				mouseStatus += MOVING_MOUSE;
			else{
				mouseStatus -= MOVING_MOUSE;
				getMap().repaint();
			}
		moving = state;
	}


/**	wireMouse
	Sets wether or not Mouse actions are ignored by this Moveable Object.
	* @ param state true if ignored, false if not ignored.
*/
	public void wireMouse(boolean state){
		if (wiring != state)
			if ( state == true)
				mouseStatus += WIRING_MOUSE;
			else
				mouseStatus -= WIRING_MOUSE;
		wiring = state;

	}


/**	ignoreMouse
	Sets wether or not Mouse actions are ignored by this Moveable Object.
	* @ param state true if ignored, false if not ignored.
*/
	public void ignoreMouse(boolean state){
		if (ignoring != state)
			if ( state == true)
				mouseStatus += IGNORE_MOUSE;
			else
				mouseStatus -= IGNORE_MOUSE;
		ignoring = state;

	}


/**	getMouseStatus
	* @return the mouseStatus of this Moveable Object.
*/
	public int getMouseStatus() {
		//System.out.println("" + mouseStatus);
		return mouseStatus;
	}



// ********************* Event METHODS
/** leftButtonPressed
  * This function is called when the left button is pressed. It sets up the icon to drag.
  * @param x is the x part new location.
  * @param y is the y part of the new location.
*/
  public void leftButtonPressed(int x,int y)
  {
    setZero(new Point (x, y));
    getMap().add(this);
    setMoving(true);
    getMap().movingThis(this);
  }

/** leftButtonRelease
  * This function changes the status back to stationary
*/
  public void leftButtonReleased()
  {
    setMoving(false);
    getMap().movingThis();
  }

/* leftMouseDragged
  * This function is called when the icon is dragged.
  * @param x is the x part new location. 
  * @param y is the y part of the new location. 
*/ 

  public void leftMouseDragged(int x, int y)
  {
    delta.x = x - getZero().x;
    delta.y = y - getZero().y;

    move(delta);
  }

// ********************* FIELDS


	public final static Color DEFAULT_COLOR	= Color.white;
	public final static int   NORMAL_MOUSE	= 0;
	public final static int   OPTION_MOUSE	= 1;
	public final static int   IGNORE_MOUSE	= 2;
	public final static int   MOVING_MOUSE	= 4;
	public final static int   WIRING_MOUSE	= 8;

	Map	map;

	private Point	locMap	= new Point(1,1);
	private Point 	zero 	= new Point(0,0);
	private Point 	delta 	= new Point(0,0);

	private boolean moving 	 = false;
	private boolean wiring 	 = false;
	private boolean ignoring = false;

	private int mouseStatus = NORMAL_MOUSE;
}
