package builder;

import java.awt.*;




/**	A moveable Label.
	* @author Arturo Falck.
	* @version 1.1 07/97
*/
public class Name extends Moveable {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param text the text of the label.
	* @param m the Map where the Name is placed.
	* @param l initial location of the Name in the Map.
*/
	public Name (String text, Map m, Point l){
		super();
		setMap(m);
		setLocMap(l);
                setListener(this);
		setLabel(text);

		int textLength 	= text.length() * 7 + 3;

		initialize(map.getBackground(), 
			   new Dimension(textLength, 12));


	}

/**	Constructor.
	* @param m the Map where the Name is placed.
	* @param l initial location of the Name in the Map.
*/
	public Name (Map m, Point l) {
		this("Label", m, l);
	}

/**	Constructor.
	* @param m the Map where the Name is placed.
*/
	public Name (Map m) {
		this(m, new Point(1,1));
	}



// ********************* METHODS


/**	setLabel
	Sets the text for this Name.
	* @param text the text of the label.
*/
	public void setLabel(String text){
		label = text;

		int textLength 	= text.length() * 7 + 3;

		initialize(map.getBackground(), 
			   new Dimension(textLength, 12));

		if (cover != null)
			cover.setText(label);
		repaint();
	}

/**	getLabel
	* @return the text of the label.
*/
	public String getLabel(){
		return toString();
	}

/**	toString
	* @return the text of the label.
*/
	public String toString(){
		return label;
	}


/**	lock
	Locks this Name in place.
	* @param status true if locked, false if unlocked.
*/
	public void lock(boolean status){
		if (status){
			if (getComponentAt(1,1) instanceof Name){
				cover = new Label(this.toString());
				cover.setBounds(0,
						0,
						this.getBounds().width,
						this.getBounds().height	);
				add(cover);
			}
		} else {
			if (getComponentAt(1,1) instanceof Label) remove(cover);
		}
	}



/**	paint
	Defines the icon for the map representation of this Node.
	* @param g the graphics context.
*/
	public void paint (Graphics g) {
		g.setColor(Color.black);
		g.drawString(label, 1, 10);
	}



// ********************* FIELDS


	private String label;
	private Label cover;
}
