package builder;
import java.io.*;
import java.util.*;
import java.net.*;
/** Author Dennis Seay
  * Version 1.1
*/

public class SaveNodes
{
               Socket       Bsocket             = null;
			   PrintWriter  outStream          = null;
               Vector       data               = null;
               Vector       wire               = null;
               FormSpace    f                  = null;
   	       WorkSpace    w                  = null;
  static final String BEGIN_SPACE_DATA         = "<---BEGIN-SPACE-DATA--->";
  static final String END_SPACE_DATA           = "<---END-SPACE-DATA--->";

/** Main Constructor
  * The constructor takes in the data to be stored.
  * @param nodes is a vector that contains all the nodes to be saved.
  * @param wires is a vector that contains all the wires to be saved.
  * @param fileName is the file to save the data to.
*/
  public SaveNodes(Vector nodes, Vector wires,String fileName, FormSpace form, WorkSpace work)
  {
    data = nodes;
    wire = wires;
    f = form;
    w = work;
	//must change to dynamically find the host
        try {
			Bsocket = new Socket("viz7.sv.vt.edu",4444);
			outStream =	new PrintWriter(Bsocket.getOutputStream(), true);
	        } catch (UnknownHostException e) {
            System.err.println("Don't know about host:.");
            System.exit(1);
			}
			catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to:.");
            System.exit(1);
			}
			outStream.println(fileName);
			
  }

/** closeSaveFile
  * This function closes any open files that were being saved to.
*/
  public void closeSaveFile()
  {
    outStream.close();
	/*need to close the socket and stuff*/
  }


/** saveLocation
  * This function saves the workspace and formspace location and size
*/
  public void saveLocation()
  {
    outStream.println(BEGIN_SPACE_DATA);
    f.save(outStream);
    w.save(outStream);
    outStream.println(END_SPACE_DATA);
  }

/** saveWire
  * This function saves the node names to connect.
*/

  public void saveWire()
  {
    outStream.println(Wire.BEGIN_WIRE_DATA);
    for(int x=0;x<wire.size();x++)
    {
      ((Wire)wire.elementAt(x)).save(outStream);
      System.out.println("Node: " + x);
    }
    outStream.println(Wire.END_WIRE_DATA);
  }

/** saveData
  * This function saves the information to recreate a node.
*/

  public void saveData()
  {
    outStream.println(Node.BEGIN_NODE_DATA);
    for(int x=0;x<data.size();x++)
    {
      ((Node)data.elementAt(x)).save(outStream,"abstract");
      System.out.println("Node: " + x);
    }
    outStream.println(Node.END_NODE_DATA);
  }
}
/** openSaveFile
  * This function opens the file for saving if it is a valid file name/file.
*/
/*  PrintWriter openSaveFile(String fileName)
  {
    PrintWriter os;
    File f = new File(fileName);
    if(f.exists())
    {
		commented out by randy
      if(f.isFile())
      {
        System.out.println("Over writing previous file");
      }else
      {
        System.out.println("Can not write to a directory");
        return null;
      }
    }
    try
    {
     os = new PrintWriter(new FileOutputStream(f),true);
    }catch(IOException e)
    {
      System.out.println("Unable to save");
      return null;
    }
    return os;
  }
}
*/
