package builder;

import java.awt.*;
import java.io.*;



/**	A Node whose GUI representation is a scroll bar similar to a number line.
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class SliderNode extends Node {



// ********************* CONSTRUCTORS


/**	Main Constructor.
*/
	SliderNode (String text, Map map, Point locMap, 
			Map form, Point locForm) {
		super(text, map, locMap, form, locForm);

		if (getMap().editMode() == false){
			runMode();
		}
		setupSlider();
	}

/** setupSlider
  * This function initializes the slider attributes.
*/
  void setupSlider()
  {
    nodeSlider.addAdjustmentListener(new NodeAdjustmentListener(this));
    nodeSlider.setMinimum(0);
    nodeSlider.setMaximum(101);	
    nodeSlider.setVisibleAmount(1); 
    getFormLabel().setVisible(false);
    setSize(getBounds().width,getBounds().height/2);
    minLabel = new Name 
      (String.valueOf(nodeSlider.getMinimum()), getMap(),
      new Point (getLocMapX(),getLocMapY()-12));
    maxLabel = new Name 
     (String.valueOf(nodeSlider.getMaximum()-1), getMap(),
     new Point (getLocMapX() + getBounds().width-20,getLocMapY()-12));
  }

/** setMin
  * This function sets the minimum value that the slider can acheive.
  * @param min is the value of the new minimum.
*/

  void setMin(int min)
  {
    nodeSlider.setMinimum(min);
    minLabel.setLabel(String.valueOf(min));
  }


/** setMax
  * This function sets the maximum value that the slider can acheive.
  * @param max is the value of the new maximum.
*/

  void setMax(int max)
  {
    nodeSlider.setMaximum(max+1);
    maxLabel.setLabel(String.valueOf(max));
  }

/**	Constructor used when placing the Node onto a location on the Form.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	public SliderNode (String text, Map map, Map form, Point locForm) {
		this(text, map, new Point(1 , 1), form, locForm);

		moveIcon(new Point(getNodeNumber() * 5 , 
				   getNodeNumber() * 5));
	}

/**	Constructor used when placing the Node onto a location on the Map.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param locMap the location of the Node's icon on the map.
	* @param form the Panel on which the Node's GUI is placed.
*/
	public SliderNode (String text, Map map, Point locMap, Map form) {
		this(text, map, locMap, form, new Point(1 , 1));

		move(new Point(	getNodeNumber() * 5 , 
				getNodeNumber() * 5));
	}



// ********************* METHODS


/**	paintPlaceHolder
	Defines the graphical representation of this Node.
	* @param g the graphics context.
*/
	public void paintPlaceHolder (Graphics g) {
		g.setColor(Color.red);
		g.drawString("<<  Slider  >>", 10, 13);
		g.draw3DRect(1, 1, 120, 14, true);
	}


/**	getComponent  
	* @param c the component.
	* @return the Component of this Node
*/
	public Component getComponent(){
		return nodeSlider;
	}


/**     move
        Moves the GUI representation of this node within the Form.
        * @param delta how much to move the place-holder
        from its present position.
*/
  public void move(Point delta)
  {
    minLabel.move(delta);
    maxLabel.move(delta);
    super.move(delta);
  }

/** getMinLabel 
  * @return the minimum label
*/
  Name getMinLabel()
  {
    return minLabel;
  }


/** getMaxLabel 
  * @return the maximum label
*/

  Name getMaxLabel()
  {
    return maxLabel;
  }

/** save
  * This function writes node information that is in required for SliderNode 
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os, String type)
  {
    super.save(os,"SliderNode");
    os.println(nodeSlider.getValue());
    os.println(minLabel.getLabel());
    os.println(minLabel.getLocMapX() + " " + minLabel.getLocMapY());
    os.println(minLabel.isVisible());
    os.println(maxLabel.getLabel());
    os.println(maxLabel.getLocMapX() + " " + maxLabel.getLocMapY());
    os.println(maxLabel.isVisible());
    os.println(END_OF_NODE);
  }



// ********************* FIELDS


	private Scrollbar nodeSlider = new Scrollbar(Scrollbar.HORIZONTAL);
	private Name minLabel = null;
	private Name maxLabel = null;
}
