package builder;

import java.awt.*;
import java.io.*;



/**	A Node whose GUI representation is a Text Field.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class TextFieldNode extends Node {



// ********************* CONSTRUCTORS


/**	Main Constructor.
*/

	TextFieldNode (String text, Map map, Point locMap, 
			Map form, Point locForm) {
		super(text, map, locMap, form, locForm);

		if (getMap().editMode() == false){
			runMode();
		}

		nodeTextField.addActionListener(new NodeActionListener(this));
	}

/**	Constructor used when placing the Node onto a location on the Form.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param form the Panel on which the Node's GUI is placed.
	* @param locForm the location of the Node's GUI on the Panel.
*/
	public TextFieldNode (String text, Map map, Map form, Point locForm) {
		this(text, map, new Point(1 , 1), form, locForm);

		moveIcon(new Point(getNodeNumber() * 5 , 
				   getNodeNumber() * 5));
	}

/**	Constructor used when placing the Node onto a location on the Map.
	* @param text the name of the Node.
	* @param map the Map where the NOde will paint itself onto.
	* @param locMap the location of the Node's icon on the map.
	* @param form the Panel on which the Node's GUI is placed.
*/
	public TextFieldNode (String text, Map map, Point locMap, Map form) {
		this(text, map, locMap, form, new Point(1 , 1));

		move(new Point(	getNodeNumber() * 5 , 
				getNodeNumber() * 5));
	}



// ********************* METHODS


/**	runMode
	* Determines if the node is in run mode (non-editable)
	* @return if the node is in run mode 
*/
	public boolean runMode(){
		if (super.runMode()){
			if (getIoOption() == OUTPUT)
				((TextField)getComponent()).setEditable(false);
		}
		return true;
	}


/**	paintPlaceHolder
	Defines the graphical representation of this Node.
	* @param g the graphics context.
*/
	public void paintPlaceHolder (Graphics g) {
		g.setColor(Color.red);
		g.drawString("<<Text Field>>", 10, 20);
		g.draw3DRect(1, 1, 120, 30, true);
	}


/**	getComponent
	* @return the Component of this Node.  
*/
	public Component getComponent(){
		return nodeTextField;
	}


/**	incommingAction
	Defines what to do when an Action reaches this Node through a wire.
	* @return true.  
	* @param caller node that broadcasted the action.
*/
	public boolean incommingAction(Node caller){
		((TextField)getComponent()).setText("" + caller.getArgument());
		return true;
	}

/** save
  * This function writes node information that is in required for the TextFieldNode
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os, String type)
  {
    super.save(os,"TextFieldNode");
    os.println(nodeTextField.getText());
    os.println(END_OF_NODE);
  }


// ********************* FIELDS


	private TextField nodeTextField = new TextField("");
}
