package builder;

import java.awt.*;
import java.awt.event.*;


/**	A ToolBar for Creating the different Nodes.
	* @author Arturo Falck.
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class ToolBar extends Map {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param flow the Map where the Icons will be placed.
	* @param form the Map where the GUI representations will be placed.
*/
	public ToolBar (Map flow, Map form) {
		super();
		this.flow=flow;
		this.form=form;

		ToolBarActionListener a = new ToolBarActionListener(this);

		setBackground(Color.lightGray);
		setBounds( 10, 50, 400, 60 );
		setLayout(null);
		add(new ToolIcon("Mode",a),85,60,2,0);
		add(new ToolIcon("Wire",a),85,60,92,0);
		add(new ToolIcon("Text Input",a),85,30,182,0);
		add(new ToolIcon("Text Output",a),85,30,182,30);
		add(new ToolIcon("Drop Box",a),85,30,272,0);
		add(new ToolIcon("Slider Bar",a),85,30,272,30);
		add(new ButtonToolIcon(a),85,30,362,0);
		add(new ToolIcon("Mail Filter",a),85,30,362,30);
	}



// ********************* METHODS


/**	add
	Adds a ToolIcon to this ToolBar.
	* @param c the the ToolIcon to be added.
*/
	public void add(ToolIcon c){
		c.setMap(this);
		if (nodesVector.isEmpty())
			c.setLocMap(new Point(2,0));
		else
			c.setLocMap(new Point(
			((ToolIcon)nodesVector.lastElement()).getLocMapX() + 90,
			((ToolIcon)nodesVector.lastElement()).getLocMapY()   ));
		c.initialize(85, 30);
		nodesVector.addElement(c);		
	}


/**	add
	Adds a ToolIcon to this ToolBar.
	* @param c the the ToolIcon to be added.
	* @param cWidth is the size of the icon.
	* @param cHeight is the height of the icon
	* @param x is one of the co-ordinate to place the icon at.
	* @param y is one of the co-ordinate to place the icon at.
*/
  public void add(ToolIcon c, int cWidth, int cHeight, int x, int y)
  {
    c.setMap(this);
    if (nodesVector.isEmpty())
    {
      c.setLocMap(new Point(x,y));
    }
    else
    {
      c.setLocMap(new Point(x,y));
    }
    c.initialize(cWidth, cHeight);
    nodesVector.addElement(c);		
  }
// ********************* Event METHODS

/** actionHelper
  * This is called by the ToolBarActionListener to execute the appropriate
  * action.
  * @param arg is the action that occured.
*/
// This code should be in the event handler. A get form and get flow method should
// be added to accomidate this.
// This is were the code to handle drag and drop components will go. djs.
	public void actionHelper(String arg){
		if (arg.equals("Button"))
		{
			new ButtonNode("Button", flow, new Point(1, 1), form);
		}
		if (arg.equals("Text Input"))
			new TextFieldNode("Input", flow, new Point(1, 1), form);
		if (arg.equals("Text Output")){
			TextFieldNode t = 
			  new TextFieldNode("Output", flow, new Point(1, 1),form);
			t.setIoOption(Node.OUTPUT);
		}
		if (arg.equals("Drop Box"))
			new PullDownNode("Pull Menu", flow, 
						new Point(1, 1), form);
		if (arg.equals("Mode")){
			if (edit == true)
				flow.editMode(edit = false);
			else
				flow.editMode(edit = true);
			repaint();
		}
		if (arg.equals("Wire"))
			flow.wireAll(true);
		if(arg.equals("Slider Bar"))
			new SliderNode("Slider",flow,new Point(1,1),form);
		if (arg.equals("Mail Filter"))
		{
			MailToNode m = new MailToNode("Mail To", flow, new Point(1, 1), form);
			m.setIoOption(Node.OUTPUT);
		}

	}

// ********************* FIELDS


	private boolean edit = true;
	public  Map flow, form;
}
