package builder;

import java.awt.*;
import java.io.*;



/**	Wire that conect Icons of two Nodes.
	* @author Arturo Falck.
	* @version 1.0 03/21/97
*/
public class Wire extends Moveable {



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* @param map the Map where this wire will be drawn.
*/
	public Wire(Node first, Node second){
		super();
		setMap (first.getIcon().getMap());
		setListener(this);
		inputNode = first;
		outputNode = second;

		initialize();
	}



// ********************* METHODS


/**	initialize
	Initializes the Wire and returns true if initialization is complete.
	* @return true.
*/
	public boolean initialize(){
//((Node)getInput()).getMap().addWire(this);  // This is a kludge that needs to be fixed in the design
		getInput().addWire(this);
		getOutput().addWire(this);

		getMap().addWire(this);
		getMap().wireAll(false);
		getMap().repaint();
		return true;
	}


/**	transmit
	Transmits the Action of the Input Node to the Output Node(s).
	* @return true if the Action is fullfilled by (all) of the Output Nodes.
*/
	public boolean transmit(){
		return getOutput().incommingAction(getInput());		
	}


/**	getInput
	* @return the Input Node to this Wire.
*/
	public Node getInput(){
		return inputNode;
	}

/**	getOutput
	* @return the Output Node(s) to this Wire.
*/
	public Node getOutput(){
		return outputNode;
	}


/**	paintWire
	Draws the wire in the Map.
	* @param g the graphics context.
*/
	public void paintWire (Graphics g) {
		Icon input  = inputNode.getIcon();
		Icon output = outputNode.getIcon();

		Point start = new Point (	input.getLocMapX()  + 
						input.getBounds().width/2,
						input.getLocMapY()  + 
						input.getBounds().height/2);
		Point end   = new Point (	output.getLocMapX() + 
						output.getBounds().width/2,
						output.getLocMapY() + 
						output.getBounds().height/2);
		g.setColor(Color.red);
		g.drawLine(start.x, start.y, end.x, end.y);

		drawArrow(g, start, end);
	}

/**	drawArrow
	Draws an Arrow at the center of the wire to indicate the direction of 
	the data flow.
	* @param g the graphics context.
	* @param start the point where the data flows FROM.
	* @param end the point where the data flows TO.
*/
	public void drawArrow (Graphics g, Point start, Point end) {
		Point middle = new Point(	(start.x + end.x) / 2,
						(start.y + end.y) / 2	);

		Point delta = new Point ( end.x - middle.x, end.y - middle.y);



		Point arrowPoint = new Point(	(middle.x + end.x) / 2,
						(middle.y + end.y) / 2	);

		Point arrowEnd1 = new Point(	middle.x - delta.y,
						middle.y + delta.x	);

		Point arrowEnd2 = new Point(	middle.x + delta.y,
						middle.y - delta.x	);

		arrowPoint = normalize(ARROW_SIZE, middle, arrowPoint);
		arrowEnd1  = normalize(ARROW_RATIO * ARROW_SIZE, 
					middle, arrowEnd1);
		arrowEnd2  = normalize(ARROW_RATIO * ARROW_SIZE, 
					middle, arrowEnd2);


		int xPoints[] = {	arrowPoint.x,
					arrowEnd1.x,
					arrowEnd2.x };
		int yPoints[] = {	arrowPoint.y,
					arrowEnd1.y,
					arrowEnd2.y };


		g.fillPolygon(xPoints, yPoints, 3);

	}

/**	normalize
	Normalices the vector defined between the center (c) point and the
	p point to a value specified by the size argument.
	* @return the new p Point such that the line c-p(new) is in the same
	direction as the line c-p(old) but of length size.
	* @param size length of the resulting line c-p.
	* @param c the "center" point for the normalization.
	* @param p the Point to be moved to a "normalized" location.
	* @return 
*/
	public Point normalize (double size, Point c, Point p){
		Point delta = new Point ( p.x - c.x, p.y - c.y);

		int normX = (int)( size * delta.x / 
				Math.sqrt(delta.x * delta.x + 
					  delta.y * delta.y));
		int normY = (int)( size * delta.y / 
				Math.sqrt(delta.x * delta.x + 
					  delta.y * delta.y));

		return new Point(c.x + normX, c.y + normY);
	}

/** save
  * This function writes node information that is in required for the Wire
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os)
  {
    os.println(inputNode.getNodeNumber() + " " + outputNode.getNodeNumber());
  }


// ********************* FIELDS


	public final static int ARROW_SIZE = 14;
	public final static double ARROW_RATIO = .4;
	public final static String BEGIN_WIRE_DATA = "<--BEGIN-WIRE-DATA-->";
	public final static String END_WIRE_DATA = "<--END-WIRE-DATA-->";

	private Node inputNode;
	private Node outputNode;
}
