package builder;

import java.awt.*;
import java.awt.event.*;
import java.io.*;


/**	Creates the workspace window for the builder interface.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/

public class WorkSpace extends MyFrame{



// ********************* CONSTRUCTORS


/**	Main Constructor.
	* The form and the workspace are created when this class is constructed.
*/
public WorkSpace (Map M,ToolBar toolBar)
{
  super("Demo: Work Space");

  MenuBar mbar = new MenuBar();
  initFileMenu(mbar);
  initEditMenu(mbar);

  M.setBounds(new Rectangle( 10, 50, 465, 200));

  initWorkSpace(toolBar,M,mbar);
  toolbar = toolBar;
}

private void initEditMenu(MenuBar mbar)
{
//---------------------------------------------------------------------------------
// The edit menu is not implemented yet. When it is implemented new actions should
// be added to the WorkSpaceActionListener class.
//---------------------------------------------------------------------------------
		edit.add(new MenuItem("Undo"));
		edit.add(new MenuItem("-"));
		edit.add(new MenuItem("Cut"));
		edit.add(new MenuItem("Copy"));
		edit.add(new MenuItem("Paste"));
		mbar.add(edit);
		edit.setEnabled(false);
}
/** initFileMenu
  * this function creates the File Menu and adds it the menubar.
  * @param mbar is the menu bar to add the menu to.
*/
private void initFileMenu(MenuBar mbar)
{
//---------------------------------------------------------------------------------
// The actions that are disabled are not handled in the WorkSpaceActionListener class.
// This functionality must be handled in the WorkSpaceActionListener class when the
// functionality is added to the system.
//---------------------------------------------------------------------------------
  MenuItem newProject = new MenuItem("New Project");
  MenuItem openProject = new MenuItem("Open Project...");
  MenuItem saveProject = new MenuItem("Save Project");
  MenuItem saveAs = new MenuItem("Save As...");
  MenuItem submit = new MenuItem("Submit...");
  file.add(newProject);
  file.add(openProject);
  file.add(saveProject);
  file.add(saveAs);
  file.add(new MenuItem("-"));
  file.add(submit);
  file.add(new MenuItem("-"));
  file.add(new MenuItem("Quit"));
  mbar.add(file);
  //openProject.setEnabled(false);
  //saveProject.setEnabled(false);
  saveAs.setEnabled(false);
  submit.setEnabled(false);
}


/** initWorkSpace
  * This function does the basic graphical layout for the workspace
  * @param toolBar is the toolbar to be added to the workspace.
  * @param M this is the map this is used to display moveable objects.
*/
private void initWorkSpace(ToolBar toolBar, Map M,MenuBar mbar)
{
  setSize(new Dimension(465, 250));
  setLocation(new Point(600, 100));
  setLayout(new BorderLayout( 5 , 5 ));
  add("North", toolBar);
  add("Center", M);
  setMenuBar(mbar);
  setListeners();
  setVisible(true);
}

/**
  * This function adds the window listener for the workspace. It also adds the action listeners
  * for the menus.
*/
  public void setListeners()
  {
    this.addWindowListener(new WorkSpaceWindowListener(this));
    WorkSpaceActionListener menuListener = new WorkSpaceActionListener(this);
    file.addActionListener(menuListener);
    edit.addActionListener(menuListener);
  }
/**
	* Creates a closed window event.
*/
  protected void quitWindowHelper()
  {
    processWindowEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSED));
  }


/** save
  * This function writes the location and size of the workspace
  * @param os is the the output stream to write the data to.
*/
  public void save(PrintWriter os)
  {
    Point p = getLocation();
    Dimension d = getSize();

    os.println(p.x + " " + p.y);
    os.println(d.height + " " + d.width);
  }

/**
	Quits program when Window is destroyed.
*/

  protected void closeWindowHelper()
  {
    //setVisible(false);
    //System.exit(0);
    parent.quitBuilder();
  }

// ********************* FIELDS

  Menu   file = new Menu("File");
  Menu   edit = new Menu("Edit");
  public PM   parent = null;
  public ToolBar toolbar = null;
}

